#version 330 core

in vec2 TexCoords;

uniform sampler2D inputTexture;
uniform vec2 texelSize;
float sigmaSpace = 3.0f;
float sigmaColor = 15.0f;

out float fragColor;

void main() {
    const int kernelRadius = 6;
    float centerValue = texture(inputTexture, TexCoords).r;

    float totalWeight = 0.0;
    float blurredValue = 0.0;

    for (int i = -kernelRadius; i <= kernelRadius; i++) {
        for (int j = -kernelRadius; j <= kernelRadius; j++) {
            vec2 offset = vec2(float(i), float(j)) * texelSize;
            vec2 sampleCoord = TexCoords + offset;

            float sampleValue = texture(inputTexture, sampleCoord).r;

            float spatialWeight = exp(-0.5 * (i * i + j * j) / (sigmaSpace * sigmaSpace));

            float colorWeight = exp(-0.5 * pow(sampleValue - centerValue, 2.0) / (sigmaColor * sigmaColor));

            float weight = spatialWeight * colorWeight;

            blurredValue += sampleValue * weight;
            totalWeight += weight;
        }
    }

    fragColor = blurredValue / totalWeight;
}
