#version 430

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout (location = 3) in vec3 tangent;

out vec3 worldPosition;
out vec3 worldNormal;
out vec2 faceUv;
out vec4 FragPosLightSpace;
out mat3 TBN;
out vec3 TangentLightPos;
out vec3 TangentViewPos;
out vec3 TangentFragPos;

uniform mat4 projectionViewMat;
uniform mat4 modelMat;
uniform vec3 lightPosition;
uniform vec3 viewPosition;

uniform mat4 lightSpaceMatrix;


void main() {
  mat3 normalMatrix = transpose(inverse(mat3(modelMat)));

  gl_Position= projectionViewMat * modelMat * vec4( position, 1.0 );
  worldNormal = normalize(normalMatrix * normal);
  worldPosition= vec3(modelMat* vec4(position, 1.0));
  faceUv= uv;
  FragPosLightSpace = lightSpaceMatrix * vec4(worldPosition, 1.0);

  vec3 T = normalize(normalMatrix * tangent);
  vec3 N = normalize(normalMatrix * normal);
  T = normalize(T - dot(T, N) * N);
  // re-orthogonalize T with respect to N (Gram-Schmidt)
  T = normalize(T - dot(T, N) * N);
  vec3 B = cross(N, T);

  TBN = transpose(mat3(T, B, N));
  TangentLightPos = TBN * lightPosition;
  TangentViewPos  = TBN * viewPosition;
  TangentFragPos  = TBN * worldPosition;

}
