#pragma once
#ifndef BUFFER_H
#define BUFFER_H

#include "Bindable.h"
#include "Utils.h"

class GpuBuffer : public Bindable {

public:
	GpuBuffer() {
		glGenBuffers(1, &bufferHandle);
	}

	virtual ~GpuBuffer() {
		glDeleteBuffers(1, &bufferHandle);
	}

	virtual void setData(const void* data, std::size_t byteCount) = 0;

protected:
	GLuint bufferHandle;
};

class VertexBuffer final : public GpuBuffer {

public:
	static void unbind() {
		glBindBuffer(GL_ARRAY_BUFFER, 0);
	}

	virtual void bind() const override {
		glBindBuffer(GL_ARRAY_BUFFER, bufferHandle);
	}

	virtual void setData(const void* data, std::size_t byteCount) override {
		bind();
		glBufferData(GL_ARRAY_BUFFER, byteCount, data, GL_STATIC_DRAW);
		unbind();
	}
};

class IndexBuffer final : public GpuBuffer {

public:
	static void unbind() {
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	}

	virtual void bind() const override {
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, bufferHandle);
	}

	virtual void setData(const void* data, std::size_t byteCount) override {
		bind();
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, byteCount, data, GL_STATIC_DRAW);
		unbind();
	}
};
#endif