#pragma once
#ifndef BILATERAL_BLUR_SHADER_H
#define BILATERAL_BLUR_SHADER_H

#include "Utils.h"
#include "Shader.h"
#include "Texture.h"
#include <glm/glm.hpp>
#include "TextureSceneColor.h"

class ShaderBilateralBlur final : public ShaderProgram {
public:
    virtual ~ShaderBilateralBlur() {}

    void setInputTexture(const TextureSceneColor& inputTexture) {
        glActiveTexture(GL_TEXTURE0 + 0);
        inputTexture.bindToUnit(inputTextureUniform, 0);
    }

    void setTexelSize(float texelWidth, float texelHeight) {
        texelSize.set(glm::vec2(texelWidth, texelHeight));
    }

private:
    friend class ShaderProgram;

    ShaderBilateralBlur(GLuint handle, Shader vert, Shader frag)
        : ShaderProgram{ handle, std::move(vert), std::move(frag) } {
        initUniforms();
    }

    ShaderBilateralBlur(GLuint handle, Shader vert, Shader frag, Shader geom)
        : ShaderProgram{ handle, std::move(vert), std::move(frag), std::move(geom) } {
        initUniforms();
    }

    void initUniforms() {
        inputTextureUniform = uniformLocation<int>("inputTexture");
        texelSize = uniformLocation<glm::vec2>("texelSize");
    }

    ShaderUniform<int> inputTextureUniform;
    ShaderUniform<glm::vec2> texelSize;
};

#endif  // BILATERAL_BLUR_SHADER_H
