#pragma once
#ifndef DEPTH_DOWNSAMPLE_SHADER_H
#define DEPTH_DOWNSAMPLE_SHADER_H

#include "Shader.h"
#include "Texture.h"

class ShaderDepthDownsampling final : public ShaderProgram {
public:
    virtual ~ShaderDepthDownsampling() {}

    void setSceneDepth(const TextureSceneDepth& depthTexture) {
        glActiveTexture(GL_TEXTURE0 + 1);
        depthTexture.bindToUnit(sceneDepthUniform, 1);
    }

    void setTexelSize(float width, float height) {
        texelSizeUniform.set(glm::vec2(1.0f / width, 1.0f / height));
    }

private:
    friend class ShaderProgram;

    ShaderDepthDownsampling(GLuint handle, Shader vert, Shader frag)
        : ShaderProgram{ handle, std::move(vert), std::move(frag) } {
        initUniforms();
    }

    ShaderDepthDownsampling(GLuint handle, Shader vert, Shader frag, Shader geom)
        : ShaderProgram{ handle, std::move(vert), std::move(frag), std::move(geom) } {
        initUniforms();
    }

    void initUniforms() {
        sceneDepthUniform = uniformLocation<int>("sceneDepth");
        texelSizeUniform = uniformLocation<glm::vec2>("texelSize");
    }

    ShaderUniform<int> sceneDepthUniform;
    ShaderUniform<glm::vec2> texelSizeUniform;
};

#endif