#pragma once
#ifndef LIGHT_UPSAMPLE_SHADER_H
#define LIGHT_UPSAMPLE_SHADER_H

#include "Shader.h"
#include "TextureSceneColor.h"

class ShaderLightUpsampling final : public ShaderProgram {
public:
    virtual ~ShaderLightUpsampling() {}

    void setLightTexture(const TextureSceneColor& lightTexture) {
        glActiveTexture(GL_TEXTURE0 + 2);
        lightTexture.bindToUnit(lightTextureUniform, 2);
    }

    void setColorTexture(const TextureSceneColor& colorTexture) {
        glActiveTexture(GL_TEXTURE0 + 10);
        colorTexture.bindToUnit(colorTextureUniform, 10);
    }

    void setTexelSize(float width, float height) {
        texelSizeUniform.set(glm::vec2(1.0f / width, 1.0f / height));
    }

private:
    friend class ShaderProgram;

    ShaderLightUpsampling(GLuint handle, Shader vert, Shader frag)
        : ShaderProgram{ handle, std::move(vert), std::move(frag) } {
        initUniforms();
    }

    ShaderLightUpsampling(GLuint handle, Shader vert, Shader frag, Shader geom)
        : ShaderProgram{ handle, std::move(vert), std::move(frag), std::move(geom) } {
        initUniforms();
    }

    void initUniforms() {
        lightTextureUniform = uniformLocation<int>("lightTexture");
        colorTextureUniform = uniformLocation<int>("colorTexture");
        texelSizeUniform = uniformLocation<glm::vec2>("texelSize");
    }

    ShaderUniform<int> lightTextureUniform;
    ShaderUniform<int> colorTextureUniform;
    ShaderUniform<glm::vec2> texelSizeUniform;
};

#endif