#pragma once
#ifndef SHADOW_MAP_SHADER_H
#define SHADOW_MAP_SHADER_H

#include "Utils.h"
#include "Shader.h"

class ShaderShadowCubemap final : public ShaderProgram  {
public:

	virtual ~ShaderShadowCubemap() {}

	void setModelMatrix(glm::mat4 modelMat) {
		modelMatUniform.set(modelMat);
	}

	void setCubefaceTransforms(std::vector<glm::mat4>& map) {
		for (int i = 0; i < 6; i++) {
			cubefaceTransforms[i].set(map[i]);
		}
	}

	void setFarPlane(float val) {
		farPlane.set(val);
	}

	void setPointLightPosition(glm::vec3 lightPos) {
		pointLightPosition.set(lightPos);
	}

private:
	friend class ShaderProgram;

	ShaderShadowCubemap(GLuint handle, Shader vert, Shader frag)
		: ShaderProgram{ handle, std::move(vert), std::move(frag) } {
		initUniforms();
	}

	ShaderShadowCubemap(GLuint handle, Shader vert, Shader frag, Shader geom)
		: ShaderProgram{ handle, std::move(vert), std::move(frag), std::move(geom) } {
		initUniforms();
	}

	void initUniforms() {
		modelMatUniform = uniformLocation<glm::mat4>("modelMat");

		for (int i = 0; i < 6; i++) {
			cubefaceTransforms.push_back(uniformLocation<glm::mat4>(("cubefaceTransforms[" + std::to_string(i) + "]").c_str()));
		}


		farPlane = uniformLocation<float>("far_plane");
		pointLightPosition = uniformLocation<glm::vec3>("lightPos");
	}

	ShaderUniform<glm::mat4> modelMatUniform;
	std::vector<ShaderUniform<glm::mat4>> cubefaceTransforms;
	ShaderUniform<float> farPlane;
	ShaderUniform<glm::vec3> pointLightPosition;
};
#endif