#pragma once
#ifndef GOD_RAY_SHADER_H
#define GOD_RAY_SHADER_H

#include "Utils.h"
#include "Shader.h"

class ShaderVolumetricGodray final : public ShaderProgram {
public:
    virtual ~ShaderVolumetricGodray() {}

    void setLightPosition(glm::vec3 lightPosScreen) {
        lightPosition.set(lightPosScreen);
    }

    void setFarPlane(float val) {
        farPlane.set(val);
    }

   void setLightSpaceMatrix(const glm::mat4 matrix) {
        lightSpaceMatrixUnitform.set(matrix);
    }

    void setDownsampledDepth(const TextureSceneDepth& downsampledDepthTexture) {
        glActiveTexture(GL_TEXTURE0 + 8);
        downsampledDepthTexture.bindToUnit(downsampledDepthTextureUniform, 8);
    }

    void setShadowCubemap(const TextureShadowCubemap& cubemapTexture) {
        glActiveTexture(GL_TEXTURE0 + 9);
        cubemapTexture.bindToUnit(shadowCubemapUniform, 9);
    }

    void setInverseViewProjection(const glm::mat4& matrix) {
        inverseViewProjectionUniform.set(matrix);
    }

    void setCameraPosition(glm::vec3 cameraPosition) {
        cameraPositionUniform.set(cameraPosition);
    }
private:
    friend class ShaderProgram;

    ShaderVolumetricGodray(GLuint handle, Shader vert, Shader frag)
        : ShaderProgram{ handle, std::move(vert), std::move(frag) } {
        initUniforms();
    }

    ShaderVolumetricGodray(GLuint handle, Shader vert, Shader frag, Shader geom)
        : ShaderProgram{ handle, std::move(vert), std::move(frag), std::move(geom) } {
        initUniforms();
    }

    void initUniforms() {
        lightSpaceMatrixUnitform = uniformLocation<glm::mat4>("lightSpaceMatrix");
        lightPosition = uniformLocation<glm::vec3>("lightPosition");
        farPlane = uniformLocation<float>("farPlane");
        downsampledDepthTextureUniform = uniformLocation<int>("downsampledDepth");
        shadowCubemapUniform = uniformLocation<int>("shadowCubemap");
        inverseViewProjectionUniform = uniformLocation<glm::mat4>("inverseViewProjection");
        cameraPositionUniform = uniformLocation<glm::vec3>("cameraPosition");
    }

    ShaderUniform<glm::mat4> lightSpaceMatrixUnitform;
    ShaderUniform<glm::vec3> cameraPositionUniform;
    ShaderUniform<glm::vec3> lightPosition;
    ShaderUniform<float> farPlane;
    ShaderUniform<int> downsampledDepthTextureUniform;
    ShaderUniform<int> shadowCubemapUniform;
    ShaderUniform<glm::mat4> inverseViewProjectionUniform;
};

#endif