#ifndef COLOR_TEXTURE_H
#define COLOR_TEXTURE_H

#include <GL/glew.h>
#include <iostream>

class TextureSceneColor : public Bindable {

public:
    static std::shared_ptr<TextureSceneColor> create(int width, int height) {
        GLuint handle;
        glGenTextures(1, &handle);
        if (!handle) {
            std::cerr << "Could not create color texture" << std::endl;
            return {};
        }

        std::shared_ptr<TextureSceneColor> texture{ new TextureSceneColor(handle, width, height) };
        glBindTexture(GL_TEXTURE_2D, handle);

        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

        return std::move(texture);
    }

    void attachToFrameBuffer(unsigned int framebuffer) const {
        glBindFramebuffer(GL_FRAMEBUFFER, framebuffer);
        glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, textureHandle, 0);
    }

    GLuint getHandle() const {
        return textureHandle;
    }

    void bind() const override {
        glBindTexture(GL_TEXTURE_2D, textureHandle);
    }

    static void unbind() {
        glBindTexture(GL_TEXTURE_2D, 0);
    }

    virtual ~TextureSceneColor() {
        if (textureHandle != -1) {
            glDeleteTextures(1, &textureHandle);
        }
    }

    void bindToUnit(ShaderUniform<int32_t>& uniform, int unit) const {
        uniform.set(unit);
        glActiveTexture(GL_TEXTURE0 + unit);
        bind();
    }

private:
    TextureSceneColor(GLuint handle, int width, int height) : textureHandle(handle), textureWidth(width), textureHeight(height) {}
    TextureSceneColor(const TextureSceneColor&) = delete;
    TextureSceneColor(TextureSceneColor&& t) : textureHandle(t.textureHandle) { t.textureHandle = -3; }

    GLuint textureHandle;
    int textureWidth, textureHeight;
};
#endif
