#ifndef DEPTH_TEXTURE_H
#define DEPTH_TEXTURE_H

#include <GL/glew.h>
#include <iostream>

class TextureSceneDepth : public Bindable {

public:
    static std::shared_ptr<TextureSceneDepth> create(int width, int height) {
        GLuint handle;
        glGenTextures(1, &handle);
        if (!handle) {
            std::cerr << "Could not create depth texture" << std::endl;
            return {};
        }

        std::shared_ptr<TextureSceneDepth> texture{ new TextureSceneDepth(handle, width, height) };
        glBindTexture(GL_TEXTURE_2D, handle);

        glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, width, height, 0, GL_DEPTH_COMPONENT, GL_FLOAT, nullptr);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

        return std::move(texture);
    }

    void attachToFrameBuffer(unsigned int framebuffer) const {
        glBindFramebuffer(GL_FRAMEBUFFER, framebuffer);
        glFramebufferTexture2D(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_TEXTURE_2D, textureHandle, 0);
    }

    GLuint getHandle() const {
        return textureHandle;
    }

    void bind() const override {
        glBindTexture(GL_TEXTURE_2D, textureHandle);
    }

    static void unbind() {
        glBindTexture(GL_TEXTURE_2D, 0);
    }

    virtual ~TextureSceneDepth() {
        if (textureHandle != -1) {
            glDeleteTextures(1, &textureHandle);
        }
    }

    void bindToUnit(ShaderUniform<int32_t>& uniform, int unit) const {
        uniform.set(unit);
        glActiveTexture(GL_TEXTURE0 + unit);
        bind();
    }

private:
    TextureSceneDepth(GLuint handle, int width, int height) : textureHandle(handle), textureWidth(width), textureHeight(height) {}
    TextureSceneDepth(const TextureSceneDepth&) = delete;
    TextureSceneDepth(TextureSceneDepth&& t) : textureHandle(t.textureHandle) { t.textureHandle = -2; }

    GLuint textureHandle;
    int textureWidth, textureHeight;
};
#endif
