#ifndef SHADOWCUBEMAPTEXTURE_H
#define SHADOWCUBEMAPTEXTURE_H
#include <iostream>
#include <ostream>
#include <GL/glew.h>

class TextureShadowCubemap : public Bindable {

public:
    static std::shared_ptr<TextureShadowCubemap> create(int depthCubemapTextureWidth, int depthCubemapTextureHeight) {
        GLuint handle;
        glGenTextures(1, &handle);
        if (!handle) {
            std::cerr << "Could not create gl texture" << std::endl;
            return {};
        }

        std::shared_ptr<TextureShadowCubemap> texture{ new TextureShadowCubemap(handle) };
        glBindTexture(GL_TEXTURE_CUBE_MAP, handle);
        for (unsigned int i = 0; i < 6; ++i) {
            glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 0, GL_DEPTH_COMPONENT, depthCubemapTextureWidth, depthCubemapTextureHeight, 0, GL_DEPTH_COMPONENT, GL_FLOAT, NULL);
        }

        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);

        return std::move(texture);
    }

    void attachToFrameBuffer(unsigned int depthMapFBO) const {

        glBindTexture(GL_TEXTURE_CUBE_MAP, textureHandle);
        glBindFramebuffer(GL_FRAMEBUFFER, depthMapFBO);
        glFramebufferTexture(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, textureHandle, 0);
        glDrawBuffer(GL_NONE);
        glReadBuffer(GL_NONE);
        glBindFramebuffer(GL_FRAMEBUFFER, 0);

    }

    virtual ~TextureShadowCubemap() {
        if (textureHandle != -1) {
            glDeleteTextures(1, &textureHandle);
        }
    }

    GLuint getHandle() const {
        return textureHandle;
    }

    void bindToUnit(ShaderUniform<int32_t>& uniform, int unit) const {
        uniform.set(unit);
        glActiveTexture(GL_TEXTURE0 + unit);
        bind();
    }

    void bind() const override {
        glBindTexture(GL_TEXTURE_CUBE_MAP, textureHandle);
    }

    static void unbind() {
        glBindTexture(GL_TEXTURE_CUBE_MAP, 0);
    }

private:
    TextureShadowCubemap(GLuint handle) : textureHandle(handle) {}
    TextureShadowCubemap(const TextureShadowCubemap&) = delete;
    TextureShadowCubemap(TextureShadowCubemap&& t) : textureHandle(t.textureHandle) { textureHandle = -1; }

    GLuint textureHandle;
};
#endif 